/*-
 * Copyright (c) 2005 Pawel Jakub Dawidek <pjd@FreeBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/soundcard.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <err.h>
#include <errno.h>

/*
 * Program takes entropy from sound device (/dev/dspW).
 * /dev/dspW uses 16-bit signed little endian encoding and we take only
 * first 8 bits from every 16-bits word.
 */

static ssize_t
compress(unsigned char *buf, ssize_t size)
{
	int i;

	for (i = 0; i < size / 2; i++)
		buf[i] = buf[i * 2];
	return (size / 2);
}

#define	MIN(a, b)	((a) < (b) ? (a) : (b))

int
main(int argc, char *argv[])
{
	unsigned long bytes;
	char buf[32];
	ssize_t len;
	int fd;

	if (argc < 3)
		errx(EXIT_FAILURE, "usage: %s device bytes", getprogname());
	fd = open(argv[1], O_RDONLY);
	if (fd == -1)
		err(EXIT_FAILURE, "Cannot open sound device");
	errno = 0;
	bytes = strtoul(argv[2], NULL, 0);
	if (errno != 0)
		err(EXIT_FAILURE, "Invalid bytes argument");
	do {
		if ((len = read(fd, buf, sizeof(buf))) == -1)
			err(EXIT_FAILURE, "Cannot read data");
		len = compress(buf, len);
		if ((size_t)len > bytes)
			len = bytes;
		write(STDOUT_FILENO, buf, len);
		bytes -= len;
	} while (bytes > 0);
	exit(EXIT_SUCCESS);
}
